@@ -64,6 +64,9 @@ typedef struct
   GtkWidget *image;
   GtkWidget *signal;
   GtkWidget *tooltip_text;
+#if GTK_CHECK_VERSION (3, 16, 0)
+  GtkCssProvider *css_provider;
+#endif
 
   XfcePanelPlugin *plugin;
   
@@ -77,8 +80,12 @@ wavelan_set_state(t_wavelan *wavelan, gint state)
 {  
   GdkRGBA color;
 #if GTK_CHECK_VERSION (3, 16, 0)
-  GtkCssProvider *css_provider;
-  gchar *css;
+  gchar *css, *color_str;
+#if GTK_CHECK_VERSION (3, 20, 0)
+  gchar * cssminsizes = "min-width: 4px; min-height: 0px";
+  if(gtk_orientable_get_orientation(GTK_ORIENTABLE(wavelan->signal)) == GTK_ORIENTATION_HORIZONTAL)
+    cssminsizes = "min-width: 0px; min-height: 4px";
+#endif
 #endif
 
   gchar signal_color_bad[] = "#e00000";
@@ -87,44 +94,40 @@ wavelan_set_state(t_wavelan *wavelan, gint state)
   gchar signal_color_strong[] = "#06c500";
   
   /* state = 0 -> no link, =-1 -> error */
-  TRACE ("Entered wavelan_set_state, state = %d", state);
+  DBG ("Entered wavelan_set_state, state = %d", state);
 
   if(state > 100)
     state = 100;
 
   wavelan->state = state;
 
-  if (state >= 1) {
+  if (state >= 1)
    gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(wavelan->signal), (gdouble) state / 100);
+  else
+   gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(wavelan->signal), 0.0);
 
-   if (wavelan->signal_colors) {
-    /* set color */
-     if (state > 70)
-      gdk_rgba_parse(&color, signal_color_strong);
-     else if (state > 55)
-      gdk_rgba_parse(&color, signal_color_good);
-     else if (state > 40)
-      gdk_rgba_parse(&color, signal_color_weak);
-     else
-      gdk_rgba_parse(&color, signal_color_bad);
+  if (wavelan->signal_colors) {
+     /* set color */
+   if (state > 70)
+    gdk_rgba_parse(&color, signal_color_strong);
+   else if (state > 55)
+    gdk_rgba_parse(&color, signal_color_good);
+   else if (state > 40)
+    gdk_rgba_parse(&color, signal_color_weak);
+   else
+    gdk_rgba_parse(&color, signal_color_bad);
 
 #if GTK_CHECK_VERSION (3, 16, 0)
+     color_str = gdk_rgba_to_string(&color);
 #if GTK_CHECK_VERSION (3, 20, 0)
-     css = g_strdup_printf("progressbar trough { min-width: 4px; min-height: 4px; } \
-                            progressbar progress { min-width: 4px; min-height: 4px; \
-                                                   background-color: %s; background-image: none; }",
+     css = g_strdup_printf("progressbar trough { %s } \
+                            progressbar progress { %s ; background-color: %s; background-image: none; }",
+                           cssminsizes, cssminsizes,
 #else
      css = g_strdup_printf(".progressbar { background-color: %s; background-image: none; }",
 #endif
-                           gdk_rgba_to_string(&color));
-     /* Setup Gtk style */
-     css_provider = gtk_css_provider_new ();
-     gtk_css_provider_load_from_data (css_provider, css, strlen(css), NULL);
-     gtk_style_context_add_provider (
-         GTK_STYLE_CONTEXT (gtk_widget_get_style_context (GTK_WIDGET (wavelan->signal))),
-         GTK_STYLE_PROVIDER (css_provider),
-         GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
-     g_free(css);
+                           color_str);
+     g_free(color_str);
 #else
      gtk_widget_override_background_color(GTK_WIDGET(wavelan->signal),
                              GTK_STATE_PRELIGHT,
@@ -136,11 +139,19 @@ wavelan_set_state(t_wavelan *wavelan, gint state)
                              GTK_STATE_SELECTED,
                              &color);
 #endif
-    }
+  } else {
+#if GTK_CHECK_VERSION (3, 20, 0)
+     /* only set size... */
+     css = g_strdup_printf("progressbar trough { %s } \
+                            progressbar progress { %s }",
+                           cssminsizes, cssminsizes);
+#endif
+  }
 
-   }
-  else
-   gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(wavelan->signal), 0.0);
+#if GTK_CHECK_VERSION (3, 16, 0)
+  gtk_css_provider_load_from_data (wavelan->css_provider, css, strlen(css), NULL);
+  g_free(css);
+#endif
 
   /* hide icon */
   if (wavelan->show_icon)
@@ -183,7 +194,15 @@ wavelan_timer(gpointer data)
       }
     }
     else {
-      wavelan_set_state(wavelan, stats.ws_quality);
+      /*
+       * Usual formula is: qual = 4 * (signal - noise)
+       * where noise is typically about -96dBm, but we don't have
+       * the actual noise value here, so approximate one.
+       */
+      if (strcmp(stats.ws_qunit, "dBm") == 0)
+        wavelan_set_state(wavelan, 4 * (stats.ws_quality - (-96)));
+      else
+        wavelan_set_state(wavelan, stats.ws_quality);
 
       if (strlen(stats.ws_netname) > 0)
         /* Translators: net_name: quality quality_unit at rate Mb/s*/
@@ -338,6 +357,11 @@ wavelan_new(XfcePanelPlugin *plugin)
 
   /* setup progressbar */
   wavelan->signal = gtk_progress_bar_new();
+  wavelan->css_provider = gtk_css_provider_new ();
+  gtk_style_context_add_provider (
+      GTK_STYLE_CONTEXT (gtk_widget_get_style_context (GTK_WIDGET (wavelan->signal))),
+      GTK_STYLE_PROVIDER (wavelan->css_provider),
+      GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
   wavelan->image = GTK_WIDGET(xfce_panel_image_new_from_source("network-wireless"));
 
   gtk_box_pack_start(GTK_BOX(wavelan->box), GTK_WIDGET(wavelan->image), FALSE, FALSE, 0);
@@ -418,14 +442,11 @@ wavelan_set_orientation(XfcePanelPlugin* plugin, GtkOrientation orientation, t_w
   gtk_orientable_set_orientation(GTK_ORIENTABLE(wavelan->box), orientation);
   gtk_orientable_set_orientation(GTK_ORIENTABLE(wavelan->signal), !orientation);
   gtk_progress_bar_set_inverted(GTK_PROGRESS_BAR(wavelan->signal), (orientation == GTK_ORIENTATION_HORIZONTAL));
-  if (orientation == GTK_ORIENTATION_HORIZONTAL) {
-   gtk_widget_set_size_request(wavelan->signal, 8, -1);
+  if (orientation == GTK_ORIENTATION_HORIZONTAL)
    gtk_widget_set_size_request(wavelan->ebox, -1, wavelan->size);
-   }
-  else {
-   gtk_widget_set_size_request(wavelan->signal, -1, 8);
+  else
    gtk_widget_set_size_request(wavelan->ebox, wavelan->size, -1);
-   }
+  wavelan_set_state(wavelan, wavelan->state);
 }
 
 static void
@@ -525,21 +546,20 @@ wavelan_create_options (XfcePanelPlugin *plugin, t_wavelan *wavelan)
 
   g_signal_connect (dlg, "response", G_CALLBACK (wavelan_dialog_response),
                     wavelan);
-  
-  gtk_container_set_border_width (GTK_CONTAINER (dlg), 2);
 
   xfce_titled_dialog_set_subtitle (XFCE_TITLED_DIALOG (dlg), _("Properties"));
               
-  vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 8);
-  gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
+  vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 6);
+  gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
   gtk_widget_show(vbox);
   gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area(GTK_DIALOG (dlg))), vbox,
                       TRUE, TRUE, 0);
   
-  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 8);
+  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 12);
   gtk_widget_show(hbox);
   
   label = gtk_label_new(_("Interface"));
+  gtk_label_set_xalign (GTK_LABEL (label), 0.0f);
   gtk_widget_show(label);
 
   interfaces = wavelan_query_interfaces ();
@@ -547,7 +567,7 @@ wavelan_create_options (XfcePanelPlugin *plugin, t_wavelan *wavelan)
   for (lp = interfaces; lp != NULL; lp = lp->next)
     gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), lp->data);
   gtk_widget_show (combo);
-  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 1);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 
   interface = gtk_bin_get_child (GTK_BIN (combo));
   if (wavelan->interface != NULL)
@@ -556,20 +576,20 @@ wavelan_create_options (XfcePanelPlugin *plugin, t_wavelan *wavelan)
       wavelan);
   gtk_widget_show(interface);
 
-  gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, FALSE, 1);
-  gtk_box_pack_start(GTK_BOX(hbox), combo, TRUE, FALSE, 1);
+  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(hbox), combo, TRUE, TRUE, 0);
 
-  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 2);
+  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 12);
   gtk_widget_show(hbox);
   autohide = gtk_check_button_new_with_mnemonic(_("_Autohide when offline"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(autohide), wavelan->autohide);
   g_signal_connect(autohide, "toggled", G_CALLBACK(wavelan_autohide_changed),
       wavelan);
   gtk_widget_show(autohide);
-  gtk_box_pack_start(GTK_BOX(hbox), autohide, TRUE, TRUE, 1);
-  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 1);
+  gtk_box_pack_start(GTK_BOX(hbox), autohide, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
   
-  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 2);
+  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 12);
   gtk_widget_show(hbox);
   autohide_missing = gtk_check_button_new_with_mnemonic(_("Autohide when no _hardware present"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(autohide_missing), 
@@ -577,18 +597,20 @@ wavelan_create_options (XfcePanelPlugin *plugin, t_wavelan *wavelan)
   g_signal_connect(autohide_missing, "toggled", 
       G_CALLBACK(wavelan_autohide_missing_changed), wavelan);
   gtk_widget_show(autohide_missing);
-  gtk_box_pack_start(GTK_BOX(hbox), autohide_missing, TRUE, TRUE, 1);
-  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 1);
+  gtk_box_pack_start(GTK_BOX(hbox), autohide_missing, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 
-  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 2);
+  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 12);
+  gtk_widget_set_margin_start (GTK_WIDGET (hbox), 12);
   gtk_widget_show(hbox);
   warn_label = gtk_label_new(_("Note: This will make it difficult to remove or configure the plugin if there is no device detected."));
   gtk_label_set_line_wrap(GTK_LABEL(warn_label), TRUE);
+  gtk_label_set_xalign (GTK_LABEL (warn_label), 0.0f);
   gtk_widget_show(warn_label);
-  gtk_box_pack_start(GTK_BOX(hbox), warn_label, TRUE, TRUE, 1);
-  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 1);
+  gtk_box_pack_start(GTK_BOX(hbox), warn_label, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 
-  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 2);
+  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 12);
   gtk_widget_show(hbox);
   signal_colors = gtk_check_button_new_with_mnemonic(_("Enable sig_nal quality colors"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(signal_colors), 
@@ -596,10 +618,10 @@ wavelan_create_options (XfcePanelPlugin *plugin, t_wavelan *wavelan)
   g_signal_connect(signal_colors, "toggled", 
       G_CALLBACK(wavelan_signal_colors_changed), wavelan);
   gtk_widget_show(signal_colors);
-  gtk_box_pack_start(GTK_BOX(hbox), signal_colors, TRUE, TRUE, 1);
-  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 1);
+  gtk_box_pack_start(GTK_BOX(hbox), signal_colors, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 
-  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 2);
+  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 12);
   gtk_widget_show(hbox);
   show_icon = gtk_check_button_new_with_mnemonic(_("Show _icon"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(show_icon), 
@@ -607,8 +629,8 @@ wavelan_create_options (XfcePanelPlugin *plugin, t_wavelan *wavelan)
   g_signal_connect(show_icon, "toggled", 
       G_CALLBACK(wavelan_show_icon_changed), wavelan);
   gtk_widget_show(show_icon);
-  gtk_box_pack_start(GTK_BOX(hbox), show_icon, TRUE, TRUE, 1);
-  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 1);
+  gtk_box_pack_start(GTK_BOX(hbox), show_icon, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 
   for (lp = interfaces; lp != NULL; lp = lp ->next)
     g_free (lp->data);
